@@ -146,6 +146,9 @@ enum litest_device_type {
 	LITEST_MOUSE_WHEEL_CLICK_ANGLE = -32,
 	LITEST_APPLE_KEYBOARD = -33,
 	LITEST_ANKER_MOUSE_KBD = -34,
+	LITEST_CYBORG_RAT = -41,
+	LITEST_YUBIKEY = -42,
+	LITEST_SYNAPTICS_I2C = -43,
 };
 
 enum litest_device_feature {
